@@ -16,7 +16,10 @@ from ulmo import util
 from ulmo import waterml
 
 
-def get_sites(wsdl_url):
+_suds_client = None
+
+
+def get_sites(wsdl_url, suds_cache=("default",)):
     """
     Retrieves information on the sites that are available from a WaterOneFlow
     service using a GetSites request.  For more detailed information including
@@ -29,22 +32,28 @@ def get_sites(wsdl_url):
         URL of a service's web service definition language (WSDL) description.
         All WaterOneFlow services publish a WSDL description and this url is the
         entry point to the service.
+    suds_cache: ``None`` or tuple
+        SOAP local cache duration for WSDL description and client object.
+        Pass a cache duration tuple like ('days', 3) to set a custom duration.
+        Duration may be in months, weeks, days, hours, or seconds.
+        If unspecified, the default duration (1 day) will be used.
+        Use ``None`` to turn off caching.
 
     Returns
     -------
     sites_dict : dict
         a python dict with site codes mapped to site information
     """
-    suds_client = suds.client.Client(wsdl_url)
+    suds_client = _get_client(wsdl_url, suds_cache)
 
     waterml_version = _waterml_version(suds_client)
     if waterml_version == '1.0':
         response = suds_client.service.GetSitesXml('')
-        response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+        response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
         sites = waterml.v1_0.parse_site_infos(response_buffer)
     elif waterml_version == '1.1':
         response = suds_client.service.GetSites('')
-        response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+        response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
         sites = waterml.v1_1.parse_site_infos(response_buffer)
 
     return dict([
@@ -53,7 +62,7 @@ def get_sites(wsdl_url):
     ])
 
 
-def get_site_info(wsdl_url, site_code):
+def get_site_info(wsdl_url, site_code, suds_cache=("default",)):
     """
     Retrieves detailed site information from a WaterOneFlow service using a
     GetSiteInfo request.
@@ -68,22 +77,28 @@ def get_site_info(wsdl_url, site_code):
         Site code of the site you'd like to get more information for. Site codes
         MUST contain the network and be of the form <network>:<site_code>, as is
         required by WaterOneFlow.
+    suds_cache: ``None`` or tuple
+        SOAP local cache duration for WSDL description and client object.
+        Pass a cache duration tuple like ('days', 3) to set a custom duration.
+        Duration may be in months, weeks, days, hours, or seconds.
+        If unspecified, the default duration (1 day) will be used.
+        Use ``None`` to turn off caching.
 
     Returns
     -------
     site_info : dict
         a python dict containing site information
     """
-    suds_client = suds.client.Client(wsdl_url)
+    suds_client = _get_client(wsdl_url, suds_cache)
 
     waterml_version = _waterml_version(suds_client)
     if waterml_version == '1.0':
         response = suds_client.service.GetSiteInfo(site_code)
-        response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+        response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
         sites = waterml.v1_0.parse_sites(response_buffer)
     elif waterml_version == '1.1':
         response = suds_client.service.GetSiteInfo(site_code)
-        response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+        response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
         sites = waterml.v1_1.parse_sites(response_buffer)
 
     if len(sites) == 0:
@@ -98,7 +113,7 @@ def get_site_info(wsdl_url, site_code):
     return site_info
 
 
-def get_values(wsdl_url, site_code, variable_code=None, start=None, end=None):
+def get_values(wsdl_url, site_code, variable_code, start=None, end=None, suds_cache=("default",)):
     """
     Retrieves site values from a WaterOneFlow service using a GetValues request.
 
@@ -122,13 +137,19 @@ def get_values(wsdl_url, site_code, variable_code=None, start=None, end=None):
     end : ``None`` or datetime (see :ref:`dates-and-times`)
         End of a date range for a query. If both start and end parameters are
         omitted, the entire time series available will be returned.
+    suds_cache: ``None`` or tuple
+        SOAP local cache duration for WSDL description and client object.
+        Pass a cache duration tuple like ('days', 3) to set a custom duration.
+        Duration may be in months, weeks, days, hours, or seconds.
+        If unspecified, the default duration (1 day) will be used.
+        Use ``None`` to turn off caching.
 
     Returns
     -------
     site_values : dict
         a python dict containing values
     """
-    suds_client = suds.client.Client(wsdl_url)
+    suds_client = _get_client(wsdl_url, suds_cache)
 
     # Note from Emilio:
     #   Not clear if WOF servers really do handle time zones (time offsets or
@@ -139,7 +160,7 @@ def get_values(wsdl_url, site_code, variable_code=None, start=None, end=None):
     #   a timezone code or offset, vs not having it (the latter, naive dt
     #   strings, is what I've been using all along)
 
-    # the intepretation of start and end time zone is server-dependent
+    # the interpretation of start and end time zone is server-dependent
     start_dt_isostr = None
     end_dt_isostr = None
     if start is not None:
@@ -154,7 +175,7 @@ def get_values(wsdl_url, site_code, variable_code=None, start=None, end=None):
         site_code, variable_code, startDate=start_dt_isostr,
         endDate=end_dt_isostr)
 
-    response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+    response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
     if waterml_version == '1.0':
         values = waterml.v1_0.parse_site_values(response_buffer)
     elif waterml_version == '1.1':
@@ -166,7 +187,7 @@ def get_values(wsdl_url, site_code, variable_code=None, start=None, end=None):
         return values
 
 
-def get_variable_info(wsdl_url, variable_code=None):
+def get_variable_info(wsdl_url, variable_code=None, suds_cache=("default",)):
     """
     Retrieves site values from a WaterOneFlow service using a GetVariableInfo
     request.
@@ -183,6 +204,12 @@ def get_variable_info(wsdl_url, variable_code=None):
         like to get more information on.  Variable codes MUST contain the
         network and be of the form <vocabulary>:<variable_code>, as is required
         by WaterOneFlow.
+    suds_cache: ``None`` or tuple
+        SOAP local cache duration for WSDL description and client object.
+        Pass a cache duration tuple like ('days', 3) to set a custom duration.
+        Duration may be in months, weeks, days, hours, or seconds.
+        If unspecified, the default duration (1 day) will be used.
+        Use ``None`` to turn off caching.
 
     Returns
     -------
@@ -191,11 +218,11 @@ def get_variable_info(wsdl_url, variable_code=None):
         `None` (default) then this will be a nested set of dicts keyed by
         <vocabulary>:<variable_code>
     """
-    suds_client = suds.client.Client(wsdl_url)
+    suds_client = _get_client(wsdl_url, suds_cache)
 
     waterml_version = _waterml_version(suds_client)
     response = suds_client.service.GetVariableInfo(variable_code)
-    response_buffer = StringIO.StringIO(response.encode('ascii', 'ignore'))
+    response_buffer = StringIO.StringIO(response.encode('utf8', 'ignore'))
 
     if waterml_version == '1.0':
         variable_info = waterml.v1_0.parse_variables(response_buffer)
@@ -220,3 +247,45 @@ def _waterml_version(suds_client):
     else:
         raise NotImplementedError(
             "only WaterOneFlow 1.0 and 1.1 are currently supported")
+
+
+def _get_client(wsdl_url, cache_duration=("default",)):
+    """
+    Open and re-use (persist) a suds.client.Client instance _suds_client throughout
+    the session, to minimize WOF server impact and improve performance.  _suds_client
+    is global in scope.
+
+    Parameters
+    ----------
+    wsdl_url : str
+        URL of a service's web service definition language (WSDL) description.
+        All WaterOneFlow services publish a WSDL description and this url is the
+        entry point to the service.
+    cache_duration: ``None`` or tuple
+        suds client local cache duration for WSDL description and client object.
+        Pass a cache duration tuple like ('days', 3) to set a custom duration.
+        Duration may be in months, weeks, days, hours, or seconds.
+        If unspecified, the suds default (1 day) will be used.
+        Use ``None`` to turn off caching.
+
+    Returns
+    -------
+    _suds_client : suds Client
+        Newly or previously instantiated (reused) suds Client object.
+    """
+    global _suds_client
+
+    # Handle new or changed client request (create new client)
+    if _suds_client is None or _suds_client.wsdl.url != wsdl_url:
+        _suds_client = suds.client.Client(wsdl_url)
+        if cache_duration is None:
+            _suds_client.set_options(cache=None)
+        else:
+            cache = _suds_client.options.cache
+            # could add some error catching ...
+            if cache_duration[0] == "default":
+                cache.setduration(days=1)
+            else:
+                cache.setduration(**dict([cache_duration]))
+
+    return _suds_client
